@@ -13,6 +13,8 @@
 #'   its neighbors. Default: 3.
 #' @param strict logical flag: if TRUE, an element must be strictly greater than
 #'   all other values in its window to be considered a peak. Default: TRUE.
+#' @param na.rm logical indicating whether \code{NA} values should be stripped
+#'   before searching for peaks.
 #'
 #' @return A vector of logical values. Values that are TRUE
 #'   correspond to local peaks in the data.
@@ -30,25 +32,36 @@
 #'
 find_peaks <-
   function(x,
-           ignore_threshold = 0.0,
+           ignore_threshold = 0,
            span = 3,
-           strict = TRUE) {
-    range_x <- range(x, finite = TRUE)
-    min_x <- range_x[1]
-    max_x <- range_x[2]
-    x <- ifelse(!is.finite(x), min_x, x)
-    # the next two lines cater for the case when max_x < 0, which is quite
-    # common with logs
-    delta <- max_x - min_x
-    top_flag <- ignore_threshold > 0.0
-    scaled_threshold <- delta * abs(ignore_threshold)
-    pks <- splus2R::peaks(x = x, span = span, strict = strict)
-    if (abs(ignore_threshold) < 1e-5)
-      return(pks)
-    if (top_flag) {
-      return(ifelse(x - min_x > scaled_threshold, pks, FALSE))
+           strict = TRUE,
+           na.rm = FALSE) {
+    # find peaks
+    if(is.null(span)) {
+      pks <- x == max(x, na.rm = na.rm)
+      if (strict && sum(pks) != 1L) {
+        pks <- logical(length(x)) # all FALSE
+      }
     } else {
-      return(ifelse(max_x - x > scaled_threshold, pks, FALSE))
+      pks <- splus2R::peaks(x = x, span = span, strict = strict)
+    }
+    # apply threshold to found peaks
+    if (abs(ignore_threshold) < 1e-5) {
+      pks
+    } else {
+      range_x <- range(x, na.rm = na.rm, finite = TRUE)
+      min_x <- range_x[1]
+      max_x <- range_x[2]
+      x <- ifelse(!is.finite(x), min_x, x)
+      # this can cater for the case when max_x < 0, as with logs
+      delta <- max_x - min_x
+      top_flag <- ignore_threshold > 0.0
+      scaled_threshold <- delta * abs(ignore_threshold)
+      if (top_flag) {
+        ifelse(x - min_x > scaled_threshold, pks , FALSE)
+      } else {
+        ifelse(max_x - x > scaled_threshold, pks , FALSE)
+      }
     }
   }
 
@@ -56,9 +69,9 @@ find_peaks <-
 #'
 #' \code{stat_peaks} finds at which x positions local y maxima are located and
 #' \code{stat_valleys} finds at which x positions local y minima are located.
-#' Both stats return x and y numeric values for peaks or valleys and formatted
-#' character labels. The formatting is determined by a format string suitable
-#' for \code{sprintf()}.
+#' Both stats return a subset of \code{data} with rows matching for peaks or
+#' valleys with formatted character labels added. The formatting is determined
+#' by a format string compatible with \code{sprintf()} or \code{strftime()}.
 #'
 #' @param mapping The aesthetic mapping, usually constructed with
 #'   \code{\link[ggplot2]{aes}} or \code{\link[ggplot2]{aes_}}. Only needs to be
@@ -100,7 +113,7 @@ find_peaks <-
 #'   converting $y$-values into character strings by means of function
 #'   \code{\link{sprintf}}.
 #'
-#' @section Computed variables:
+#' @section Returned and computed variables:
 #' \describe{
 #'   \item{x}{x-value at the peak (or valley) as numeric}
 #'   \item{y}{y-value at the peak (or valley) as numeric}
@@ -117,28 +130,55 @@ find_peaks <-
 #'
 #' @note These stats check the scale of the \code{x} aesthetic and if it is Date
 #'   or Datetime they correctly generate the labels by transforming the numeric
-#'   x values to POSIXct objects, in which case the \code{x.label.fmt} must be
-#'   suitable for \code{strftime()} rather than for \code{sprintf()}.
-#' These stats work nicely together with geoms
-#'   \code{\link[ggrepel]{geom_text_repel}} and
+#'   \code{x} values to Date or POSIXct objects, respectively. In which case the
+#'   \code{x.label.fmt} must follow the syntax supported by \code{strftime()}
+#'   rather than by \code{sprintf()}. These stats work nicely together with
+#'   geoms \code{\link[ggrepel]{geom_text_repel}} and
 #'   \code{\link[ggrepel]{geom_label_repel}} from package
 #'   \code{\link[ggrepel]{ggrepel}} to solve the problem of overlapping labels
 #'   by displacing them. Alternatively, to discard overlapping labels use
-#'   \code{check_overlap = TRUE} as argument to \code{geom_text}.
-#'  By default the labels are character values suitable to be plotted as is, but
-#'  with a suitable \code{label.fmt} labels suitable for parsing by the geoms
-#'  (e.g. into expressions containing Greek letters, super- or subscripts,
-#'  maths symbols or maths constructs) can be also easily obtained.
+#'   \code{check_overlap = TRUE} as argument to \code{geom_text}. By default the
+#'   labels are character values suitable to be plotted as is, but with a
+#'   suitable format passed as argument to \code{label.fmt} labels suitable for
+#'   parsing by the geoms (e.g. into expressions containing Greek letters,
+#'   super- or subscripts, maths symbols or maths constructs) can be also easily
+#'   obtained.
 #'
 #' @examples
-#' library(ggplot2)
-#' lynx.df <- data.frame(year = as.numeric(time(lynx)), lynx = as.matrix(lynx))
-#' ggplot(lynx.df, aes(year, lynx)) + geom_line() +
+#' # lynx is a time.series object
+#' lynx_num.df <-
+#'   try_tibble(lynx,
+#'              col.names = c("year", "lynx"),
+#'              as.numeric = TRUE) # years -> as numeric
+#'
+#' ggplot(lynx_num.df, aes(year, lynx)) +
+#'   geom_line() +
 #'   stat_peaks(colour = "red") +
 #'   stat_valleys(colour = "blue")
-#' ggplot(lynx.df, aes(year, lynx)) + geom_line() +
+#' ggplot(lynx_num.df, aes(year, lynx)) +
+#'   geom_line() +
 #'   stat_peaks(colour = "red") +
 #'   stat_peaks(colour = "red", geom = "rug")
+#' ggplot(lynx_num.df, aes(year, lynx)) +
+#'   geom_line() +
+#'   stat_peaks(colour = "red") +
+#'   stat_peaks(colour = "red", geom = "text", hjust = -0.1, angle = 33)
+#'
+#' lynx_datetime.df <-
+#'    try_tibble(lynx,
+#'               col.names = c("year", "lynx")) # years -> POSIXct
+#' ggplot(lynx_datetime.df, aes(year, lynx)) +
+#'   geom_line() +
+#'   stat_peaks(colour = "red") +
+#'   stat_valleys(colour = "blue")
+#' ggplot(lynx_datetime.df, aes(year, lynx)) +
+#'   geom_line() +
+#'   stat_peaks(colour = "red") +
+#'   stat_peaks(colour = "red",
+#'              geom = "text",
+#'              hjust = -0.1,
+#'              x.label.fmt = "%Y",
+#'              angle = 33)
 #'
 #' @export
 #' @family peaks and valleys functions
@@ -192,8 +232,7 @@ peaks_compute_group_fun <- function(data,
   } else if (is.null(y.label.fmt)) {
     y.label.fmt <- "%.4g"
   }
-  if (inherits(scales$x, c("ScaleContinuousDatetime",
-                           "ScaleContinuousDate"))) {
+  if (inherits(scales$x, "ScaleContinuousDatetime")) {
     as_label <- function(fmt, x) {
       x <- as.POSIXct(x,
                       origin = lubridate::origin)
@@ -202,6 +241,14 @@ peaks_compute_group_fun <- function(data,
     if (is.null(x.label.fmt)) {
       x.label.fmt <- "%Y-%m-%d"
     }
+  } else if (inherits(scales$x, "ScaleContinuousDate")) {
+    as_label <- function(fmt, x) {
+      x <- as.Date(x, origin = lubridate::origin)
+      strftime(x, fmt)
+    }
+    if (is.null(x.label.fmt)) {
+      x.label.fmt <- "%Y-%m-%d"
+    }
   } else {
     as_label <- function(fmt, x) {
       sprintf(fmt, x)
@@ -216,7 +263,8 @@ peaks_compute_group_fun <- function(data,
     peaks.df <- data[find_peaks(data$y,
                                 span = span,
                                 ignore_threshold = ignore_threshold,
-                                strict = strict), , drop = FALSE]
+                                strict = strict,
+                                na.rm = TRUE), , drop = FALSE]
   }
   peaks.df[["x.label"]] <- as_label(x.label.fmt, peaks.df[["x"]])
   peaks.df[["y.label"]] <- sprintf(y.label.fmt, peaks.df[["y"]])
@@ -251,8 +299,7 @@ valleys_compute_group_fun <- function(data,
   } else if (is.null(y.label.fmt)) {
     y.label.fmt <- "%.4g"
   }
-  if (inherits(scales$x, c("ScaleContinuousDatetime",
-                           "ScaleContinuousDate"))) {
+  if (inherits(scales$x, "ScaleContinuousDatetime")) {
     as_label <- function(fmt, x) {
       x <- as.POSIXct(x,
                       origin = lubridate::origin)
@@ -261,6 +308,14 @@ valleys_compute_group_fun <- function(data,
     if (is.null(x.label.fmt)) {
       x.label.fmt <- "%Y-%m-%d"
     }
+  } else if (inherits(scales$x, "ScaleContinuousDate")) {
+    as_label <- function(fmt, x) {
+      x <- as.Date(x, origin = lubridate::origin)
+      strftime(x, fmt)
+    }
+    if (is.null(x.label.fmt)) {
+      x.label.fmt <- "%Y-%m-%d"
+    }
   } else {
     as_label <- function(fmt, x) {
       sprintf(fmt, x)
@@ -275,7 +330,8 @@ valleys_compute_group_fun <- function(data,
     valleys.df <- data[find_peaks(-data$y,
                                   span = span,
                                   ignore_threshold = ignore_threshold,
-                                  strict = strict), , drop = FALSE]
+                                  strict = strict,
+                                  na.rm = TRUE), , drop = FALSE]
   }
   valleys.df[["x.label"]] <- as_label(x.label.fmt, valleys.df[["x"]])
   valleys.df[["y.label"]] <- sprintf(y.label.fmt, valleys.df[["y"]])
